import threading
import time
import requests
from .config import CODEW_API_URL


def singleton(class_):
    instances = {}

    def getinstance(*args, **kwargs):
        if class_ not in instances:
            instances[class_] = class_(*args, **kwargs)
        return instances[class_]

    return getinstance


@singleton
class KXYState:

    def __init__(self):
        self.enabled_kxy = "not_initialized"
        self.performance_enabled = False
        self.memory_enabled = False
        self.logging_enabled = False
        self.token_secret = ""
        self.codew_api = CODEW_API_URL
        threading.Thread(target=self.loop, daemon=True).start()

    def get_secret(self):
        return self.token_secret

    def set_secret(self, secret):
        self.token_secret = secret
        self.check_token()

    def loop(self):
        while True:
            if not self.token_secret:
                time.sleep(0.5)
                continue
            self.check_token()
            time.sleep(2)

    def check_token(self):
        if self.enabled_kxy != "not_initialized":
            return

        base_url = self.codew_api + "/kxy/status"
        try:
            response = requests.post(base_url, headers={"secret": self.token_secret}, timeout=2, verify=False)
            response.raise_for_status()
            self.enabled_kxy = response.text.strip()
            if self.enabled_kxy == "1":
                self.performance_enabled = True

        except Exception as e:
            print("Error checking KXY status:", e)

    def parse_app_config(self):
        ...





def mem_profile(func):
    def wrapper(*args, **kwargs):
        global kxy_enabled
        if kxy_enabled == "1":
            # TODO: Implement memory reporting
            print("Memory profiling is not implemented yet.. we're working on it.")
            result = func(*args, **kwargs)
            return result
        else:
            return func(*args, **kwargs)

    wrapper.__name__ = func.__name__
    wrapper.__doc__ = func.__doc__
    return wrapper




import cProfile
import tempfile
import requests
import threading
import atexit
import time

from .config import CODEW_API_URL
from .state import KXYState

__profiler = cProfile.Profile()
kxy_state = KXYState()


def push_results(profiler):
    try:
        upload_url = CODEW_API_URL + "/kxy/upload"
        with tempfile.NamedTemporaryFile(suffix=".prof", delete=False) as tmp:
            profiler.dump_stats(tmp.name)
            tmp.seek(0)
            files = {"file": ("profile.prof", tmp, "application/octet-stream")}
            response = requests.post(upload_url, files=files, headers={"secret": kxy_state.get_secret()}, timeout=5, verify=False)
            if response.status_code != 200:
                print(f"Failed to send profile: {response.status_code} - {response.text}")
                return None
            return response.json().get("id")
    except requests.RequestException as e:
        print("Error connecting to 7176")


def profile(func):
    def wrapper(*args, **kwargs):
        print(f"Profiling {func.__name__}...")
        if kxy_state.performance_enabled:
            _profiler = cProfile.Profile()
            _profiler.enable()
            try:
                result = func(*args, **kwargs)
                _profiler.disable()
            except Exception as e:
                _profiler.disable()
                raise e
            finally:
                print("Profiling complete.")
                push_results(_profiler)
            return result
        else:
            return func(*args, **kwargs)

    wrapper.__name__ = func.__name__
    wrapper.__doc__ = func.__doc__
    return wrapper


def end_kxy(_time=None):
    global __profiler
    if _time:
        time.sleep(_time)
    __profiler.disable()
    push_results(__profiler)


def pause_profiling():
    global __profiler
    __profiler.disable()


def resume_profiling():
    global __profiler
    __profiler.enable()


def init_kxy(_time=None):
    global __profiler
    __profiler.enable()
    if _time:
        end_thread = threading.Thread(target=end_kxy, args=(_time,), daemon=True)
        end_thread.start()
    atexit.register(end_kxy, None)



import sys
import threading
import asyncio
import traceback

import requests

from src.kxy_codew.state import KXYState
from src.kxy_codew.config import CODEW_API_URL

kxy_state = KXYState()


def push_results(root_cause, traceback_str):
    try:
        upload_url = CODEW_API_URL + "/kxy/watchman"
        response = requests.post(upload_url, json={"root_cause": root_cause, "content": traceback_str}, headers={"secret": kxy_state.get_secret()}, timeout=5, verify=False)
        if response.status_code != 200:
            print(f"[watchman] Failed to send error: {response.status_code} - {response.text}")
            return None
        return response.json().get("id")
    except requests.RequestException as e:
        print("[watchman] Error connecting to 7176")



def wake_up():

    def format_exception(exc_type, exc_value, exc_traceback):
        tb_lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
        # Take the last meaningful line for location
        last_line = tb_lines[-1].strip()  # e.g., 'NameError: name 'load_video_filew' is not defined'
        # Get the filename and line number from traceback
        tb_summary = traceback.extract_tb(exc_traceback)
        if tb_summary:
            last_frame = tb_summary[-1]
            filename = last_frame.filename
            lineno = last_frame.lineno
            funcname = last_frame.name
            root_cause = f"{exc_type.__name__}: {exc_value} in File \"{filename}\", line {lineno}, in {funcname}"
        else:
            root_cause = last_line

        return root_cause, "\n".join(tb_lines)

    def log_exception(exc_type, exc_value, exc_traceback):
        print("[watchman] Unhandled Exception")
        if issubclass(exc_type, KeyboardInterrupt):
            sys.__excepthook__(exc_type, exc_value, exc_traceback)
            return
        format_exception_str, traceback_str = format_exception(exc_type, exc_value, exc_traceback)
        push_results(format_exception_str, traceback_str)
        traceback.print_exception(exc_type, exc_value, exc_traceback)

    sys.excepthook = log_exception

    if hasattr(threading, 'excepthook'):
        threading.excepthook = lambda args: log_exception(
            args.exc_type, args.exc_value, args.exc_traceback
        )

    try:
        loop = asyncio.get_event_loop()
        loop.set_exception_handler(
            lambda loop, context: log_exception(
                type(context.get("exception")),
                context.get("exception"),
                context.get("exception").__traceback__
            ) if context.get("exception") else print(f"[watchman] Unhandled async error: {context}")
        )
    except RuntimeError:
        print("[watchman] Unhandled RUNTIME Exception")
        pass
